/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

#ifndef INCLUDED_CPPU_ENVGUARDS_HXX
#define INCLUDED_CPPU_ENVGUARDS_HXX

#include "uno/environment.hxx"


namespace cppu
{
    /** Environment Guard
        The provided Environment becomes entered in the constructor and left
        in the destructor.
        (http://wiki.openoffice.org/wiki/Uno/Cpp/Spec/Environment_Guard)


    */
    class EnvGuard
    {
        css::uno::Environment m_env;

    public:
        explicit EnvGuard(css::uno::Environment const & env)
        {
            if (env.is())
            {
                m_env = css::uno::Environment::getCurrent();
                env.enter();
            }
        }

        ~EnvGuard()
        {
            m_env.enter();
        }

        /** Checks if the associated environment is non empty.

            @return  0 == empty, 1 == non empty
        */
        bool SAL_CALL is() const
        {
            return m_env.is();
        }

        /** Leaves the associated environment and clears
            the reference.
        */
        void clear()
        {
            if (m_env.is())
            {
                m_env.enter();
                m_env.clear();
            }
        }
    };

    /** Environment Anti-Guard
        Any entered Environment becomes left in the constructor and re-entered
        in the destructor.
        (http://wiki.openoffice.org/wiki/Uno/Cpp/Spec/Environment_AntiGuard)


    */
    class AntiEnvGuard
    {
        css::uno::Environment m_env;

    public:
        explicit AntiEnvGuard()
            : m_env(css::uno::Environment::getCurrent())
        {
            uno_Environment_enter(NULL);
        }

        ~AntiEnvGuard()
        {
            m_env.enter();
        }
    };
}

#endif

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
