/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

#ifndef INCLUDED_CPPUHELPER_WEAKREF_HXX
#define INCLUDED_CPPUHELPER_WEAKREF_HXX

#include "sal/config.h"

#include <cstddef>

#include "com/sun/star/uno/Reference.hxx"
#include "com/sun/star/uno/XInterface.hpp"
#include "cppuhelper/cppuhelperdllapi.h"


namespace com
{
namespace sun
{
namespace star
{
namespace uno
{

class OWeakRefListener;
class XWeak;

/** The WeakReferenceHelper holds a weak reference to an object.

    That object must implement the css::uno::XWeak interface.

    The WeakReferenceHelper itself is *not* thread safe, just as
    Reference itself isn't, but the implementation of the listeners etc.
    behind it *is* thread-safe, so multiple threads can have their own
    WeakReferences to the same XWeak object.
*/
class CPPUHELPER_DLLPUBLIC WeakReferenceHelper
{
public:
    /** Default ctor.  Creates an empty weak reference.
    */
    WeakReferenceHelper()
        : m_pImpl( NULL )
        {}

    /** Copy ctor.  Initialize this reference with the same interface as in rWeakRef.

        @param rWeakRef another weak ref
    */
    WeakReferenceHelper( const WeakReferenceHelper & rWeakRef );

#if defined LIBO_INTERNAL_ONLY
    WeakReferenceHelper(WeakReferenceHelper && other) noexcept : m_pImpl(other.m_pImpl)
    { other.m_pImpl = nullptr; }
#endif

    /** Initialize this reference with the hard interface reference xInt. If the implementation
        behind xInt does not support XWeak or xInt is null then this reference will be null.

        @param xInt another hard interface reference
    */
    WeakReferenceHelper( const css::uno::Reference< css::uno::XInterface > & xInt );

#if defined LIBO_INTERNAL_ONLY
    /** Initialize this reference with the hard interface reference xWeak. This
         is faster than the XInterface constructor because we can skip doing an
         UNO_QUERY.

        @param xWeak another hard interface reference
    */
    WeakReferenceHelper( const css::uno::Reference< css::uno::XWeak > & xWeak );
#endif

    /** Releases this reference.
    */
    ~WeakReferenceHelper();

    /** Releases this reference and takes over rWeakRef.

        @param rWeakRef another weak ref
    */
    WeakReferenceHelper & SAL_CALL operator = ( const WeakReferenceHelper & rWeakRef );

#if defined LIBO_INTERNAL_ONLY
    WeakReferenceHelper & operator =(WeakReferenceHelper && other);
#endif

    /** Releases this reference and takes over hard reference xInt.
        If the implementation behind xInt does not support XWeak
        or XInt is null, then this reference is null.

        @param xInt another hard reference
    */
    WeakReferenceHelper & SAL_CALL operator = (
            const css::uno::Reference< css::uno::XInterface > & xInt );

#if defined LIBO_INTERNAL_ONLY
    /** Releases this reference and takes over hard reference xWeak. This
         is faster than the XInterface constructor because we can skip doing an
         UNO_QUERY.

        @param xWeak another hard reference
    */
    WeakReferenceHelper & operator = (
            const css::uno::Reference< css::uno::XWeak > & xWeak );
#endif

    /** Returns true if both weak refs reference to the same object.

        @param rObj another weak ref
        @return true, if both weak refs reference to the same object.
    */
    bool SAL_CALL operator == ( const WeakReferenceHelper & rObj ) const
        { return (get() == rObj.get()); }

    /**  Gets a hard reference to the object.

         @return hard reference or null, if the weakly referenced interface has gone
    */
    css::uno::Reference< css::uno::XInterface > SAL_CALL get() const;

    /**  Gets a hard reference to the object.

         @return hard reference or null, if the weakly referenced interface has gone
    */
    SAL_CALL operator Reference< XInterface > () const
        { return get(); }

    /** Releases this reference.


    */
    void SAL_CALL clear();

protected:
    /// @cond INTERNAL
    OWeakRefListener * m_pImpl;
    /// @endcond
};

/** The WeakReference<> holds a weak reference to an object.

    That object must implement the css::uno::XWeak interface.

    The WeakReference itself is *not* thread safe, just as
    Reference itself isn't, but the implementation of the listeners etc.
    behind it *is* thread-safe, so multiple threads can have their own
    WeakReferences to the same XWeak object.

    @tparam interface_type type of interface
*/
template< class interface_type >
class SAL_WARN_UNUSED WeakReference : public WeakReferenceHelper
{
public:
    /** Default ctor.  Creates an empty weak reference.
    */
    WeakReference()
        : WeakReferenceHelper()
        {}

    /** Copy ctor.  Initialize this reference with a hard reference.

        @param rRef another hard ref
    */
    WeakReference( const Reference< interface_type > & rRef )
        : WeakReferenceHelper( rRef )
        {}

    /** Releases this reference and takes over hard reference xInt.
        If the implementation behind xInt does not support XWeak
        or XInt is null, then this reference is null.

        @param xInt another hard reference


    */
    WeakReference & SAL_CALL operator = (
            const css::uno::Reference< interface_type > & xInt )
        { WeakReferenceHelper::operator=(xInt); return *this; }

#if defined LIBO_INTERNAL_ONLY
    /** Releases this reference and takes over hard reference xWeak. This
         is faster than the XInterface constructor because we can skip doing an
         UNO_QUERY.

        @param xWeak another hard reference
    */
    WeakReference & operator = (
            const css::uno::Reference< css::uno::XWeak > & xWeak )
        { WeakReferenceHelper::operator=(xWeak); return *this; }
#endif

    /**  Gets a hard reference to the object.

         @return hard reference or null, if the weakly referenced interface has gone
    */
    SAL_CALL operator Reference< interface_type > () const
        { return Reference< interface_type >::query( get() ); }
};

}
}
}
}

#endif

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
